#ifndef ppio_h
#define ppio_h

#include "byteque.h"

#define WTICKS 18  /* allow 1 second for write */
#define RTICKS  4  /* allow 2 ticks for read   */

typedef enum {
  PP_UNKNOWN,       /* not yet defined */
  PP_NYBBLE,
  PP_BYTE
} PPMODE;

/*
  recieve states
*/
typedef enum {
  RCV_FINISHED = 0,
  RCV_WAKEUP,
  RCV_HEADER,
  RCV_DATA,
  RCV_CHECK
} RCVSTATE;


typedef struct tagLENGTHBLOCK {
  unsigned length;
  unsigned notLen;
} LENGTHBLOCK;

typedef struct tagPP {
  unsigned  basePort;    /* parallel base port */
  unsigned  irq;         /* irq #              */

  PPMODE   mode;        /* xfer mode for writing */

  void interrupt (*old)(); /* original IRQ handler                    */
  int	    handlerNo;     /* handler function ID 	              */

  int       lock;        /* this prevents stack overflow on multiple
                            interrupts */

  BYTEQUEUE rcv;

  void     *userData;
} PP;

/*
  lpt        = port to use, 1 <= lpt <= 3
  irq        = irq override, -1 = use default
  basePort   = base port override, -1 = use default
  rcvBufSize = # of bytes for recieve queue
               pass a negative number to not allow interrupt driven
               recieves
*/
PP      *PP_Open(int lpt, int irq, int baseport, int rcvBufSize, PPMODE mode);
void     PP_Close(PP *info);

int      PP_Write(PP *info, void *buf, int len);
int      PP_Read(PP *info, void *buf, int len);

#define  PP_RecievePending(i) ((i)->rcv.inQueue)
#define  PP_ClearQueue(i)     ((i)->rcv.inQueue = 0)

#endif
